// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Flower.java
// A flower that branches off and is very very pretty! Worst. Comment. Ever.
// ------------------------------------------------------------------------------

class Flower {
  
  Point       root;                                  // The position of the flower's root
  ArrayList   points = new ArrayList();              // The list of points in this flower
  Flower      child = null;                          // The flower's child
  float       livelihood;                            // How alive is the flower?
  boolean     primary = false;                       // Is this flower being followed by the camera?
  float       bendingSpeed = 0;                      // How fast the stem is bending
  
  /**
   * Constructor.
   */
  Flower(Point R, Vector T) {
    
    // Set the root
    root = R;
    
    // Initialise the array
    points.add(root);
    points.add(root.add(T.div(T.length()).mul(STEM_PART_LENGTH)));
    
    // Start the flower
    livelihood = 100;
  }
  
  /**
   * Executes a frame of the world and returns 
   * the running status of the flower (dead or alive?)
   */
  boolean frame() {
    
    if (livelihood >= 0) {
      
      Point beforeLast       = (Point) points.get(points.size()-2);
      Point last             = (Point) points.get(points.size()-1);
      Vector trajectory      = new Vector(beforeLast, last);
      bendingSpeed           = bendingSpeed + ((float) (STEM_ROTATION*(Math.random()-0.5)*Math.PI/180))*STEM_BENDING_INERTIA;
      Vector newTrajectory   = trajectory.rotate(bendingSpeed);
      bendingSpeed           = bendingSpeed * STEM_BENDING_DAMPING;
      
      Point newPoint = last.add(newTrajectory);
      points.add(newPoint);
      
      if (primary) {
        Vector translation = new Vector(CAMERA_POSITION, newPoint);
        CAMERA_VELOCITY = CAMERA_VELOCITY.add(translation.mul(0.005));
        CAMERA_POSITION = CAMERA_POSITION.add(CAMERA_VELOCITY.mul(1));
        CAMERA_VELOCITY = CAMERA_VELOCITY.mul(0.9);
        CAMERA_UPDATED  = true;
      }
      
      // Branch off a new flower
      if (Math.random() < STEM_BRANCH_RATE && child == null) {
        child = new Flower(newPoint, newTrajectory.rotate(((float) (STEM_ROTATION*(Math.random()-0.5)*Math.PI/180))));
        if (primary) {
          child.primary = true;
          primary = false;
        }
      }
    }
    
    // Draw the flower
    draw();
    
    return (livelihood >= 0);
  }
  
  /**
   * Draws the flower.
   */
  void draw() {
    
    // The thickness of each part of the stem
    float thickness = INITAL_STEM_THICKNESS;
   
    for (int i=1; i<points.size(); i++) {
      
      // Set the colours
      float r = (COLOUR_R-255)*livelihood/100 + 255;
      float g = (COLOUR_G-255)*livelihood/100 + 255;
      float b = (COLOUR_B-255)*livelihood/100 + 255;
      stroke(r,g,b,255);
      
      // Draw the stem part
      strokeWeight(thickness);
      Point prevPoint = (Point) points.get(i-1);
      Point currPoint = (Point) points.get(i);
      line(prevPoint.x - CAMERA_POSITION.x + 400, prevPoint.y - CAMERA_POSITION.y + 210, 
           currPoint.x - CAMERA_POSITION.x + 400, currPoint.y - CAMERA_POSITION.y + 210);
      
      // Update the thickness
      thickness -= STEM_SHRINK_RATE;
      
      // This flower is dead
      if (thickness <= 1) {
        thickness = 0;
        livelihood -= 0.1;
      }
    }
    
    // Draw the child (if any)
    if (child != null) {
      child.frame();
    }
  }
}
