import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class GrowthV2 extends PApplet {// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Shader.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// Global variables
Point    CAMERA_POSITION;
Vector   CAMERA_VELOCITY;
Vector   CAMERA_ORIENTATION;
boolean  CAMERA_UPDATED;
World    WORLD;

// The scrollers
HScroller Scroller1;
HScroller Scroller2;
HScroller Scroller3;
HScroller Scroller4;
HScroller Scroller5;

// Settings
float INITAL_STEM_THICKNESS         = 5.00f;
float STEM_SHRINK_RATE              = 0.05f;
float STEM_PART_LENGTH              = 2.00f;
float STEM_ROTATION                 = 35.0f;
float STEM_BRANCH_RATE              = 0.10f;
float STEM_BENDING_INERTIA          = 1.00f;
float STEM_BENDING_DAMPING          = 0.00f;
float COLOUR_R                      = 0.00f;
float COLOUR_G                      = 0.00f;
float COLOUR_B                      = 0.00f;
float COLOUR_R_T                    = 203.00f;
float COLOUR_G_T                    = 10.00f;
float COLOUR_B_T                    = 28.00f;

/**
 * Gets executed once when the program
 * starts.
 */
public void setup() {
  
  // Set the size and the framerate
  // and make sure all graphics are
  // smoothed out
  size(800,420);
  frameRate(30);
  smooth();
  
  // Set the proce55ing settings
  ellipseMode(CENTER);
  
  // Initialise the experiment's global variables
  CAMERA_POSITION = new Point(400, 210);
  CAMERA_VELOCITY = new Vector(0,0);
  CAMERA_ORIENTATION = new Vector(0,1);
  WORLD = new World();
  
  // Create the scrollers
  Scroller1 = new HScroller(5, 90, 35, 700, 290, 50);
  Scroller2 = new HScroller(0.0f, 1.0f, 1.0f, 700, 310, 50);
  Scroller3 = new HScroller(0.0f, 1.0f, 0.0f, 700, 330, 50);
  Scroller4 = new HScroller(2.0f, 10, 2.0f, 700, 350, 50);
  Scroller5 = new HScroller(0.0f, 0.3f, 0.1f, 700, 370, 50);
}

/**
 * Gets executed every frame.
 */
public void draw() {
  
  // Draw a purdy background
  background(255,255,255);
  
  // Update the variables
  STEM_ROTATION         = Scroller1.update();
  STEM_BENDING_INERTIA  = Scroller2.update();
  STEM_BENDING_DAMPING  = Scroller3.update();
  STEM_PART_LENGTH      = Scroller4.update();
  STEM_BRANCH_RATE      = Scroller5.update();
  
  // Execute a frame of the world
  WORLD.frame();
}


// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Flower.java
// A flower that branches off and is very very pretty! Worst. Comment. Ever.
// ------------------------------------------------------------------------------

class Flower {
  
  Point       root;                                  // The position of the flower's root
  ArrayList   points = new ArrayList();              // The list of points in this flower
  Flower      child = null;                          // The flower's child
  float       livelihood;                            // How alive is the flower?
  boolean     primary = false;                       // Is this flower being followed by the camera?
  float       bendingSpeed = 0;                      // How fast the stem is bending
  
  /**
   * Constructor.
   */
  Flower(Point R, Vector T) {
    
    // Set the root
    root = R;
    
    // Initialise the array
    points.add(root);
    points.add(root.add(T.div(T.length()).mul(STEM_PART_LENGTH)));
    
    // Start the flower
    livelihood = 100;
  }
  
  /**
   * Executes a frame of the world and returns 
   * the running status of the flower (dead or alive?)
   */
  public boolean frame() {
    
    if (livelihood >= 0) {
      
      Point beforeLast       = (Point) points.get(points.size()-2);
      Point last             = (Point) points.get(points.size()-1);
      Vector trajectory      = new Vector(beforeLast, last);
      bendingSpeed           = bendingSpeed + ((float) (STEM_ROTATION*(Math.random()-0.5f)*Math.PI/180))*STEM_BENDING_INERTIA;
      Vector newTrajectory   = trajectory.rotate(bendingSpeed);
      bendingSpeed           = bendingSpeed * STEM_BENDING_DAMPING;
      
      Point newPoint = last.add(newTrajectory);
      points.add(newPoint);
      
      if (primary) {
        Vector translation = new Vector(CAMERA_POSITION, newPoint);
        CAMERA_VELOCITY = CAMERA_VELOCITY.add(translation.mul(0.005f));
        CAMERA_POSITION = CAMERA_POSITION.add(CAMERA_VELOCITY.mul(1));
        CAMERA_VELOCITY = CAMERA_VELOCITY.mul(0.9f);
        CAMERA_UPDATED  = true;
      }
      
      // Branch off a new flower
      if (Math.random() < STEM_BRANCH_RATE && child == null) {
        child = new Flower(newPoint, newTrajectory.rotate(((float) (STEM_ROTATION*(Math.random()-0.5f)*Math.PI/180))));
        if (primary) {
          child.primary = true;
          primary = false;
        }
      }
    }
    
    // Draw the flower
    draw();
    
    return (livelihood >= 0);
  }
  
  /**
   * Draws the flower.
   */
  public void draw() {
    
    // The thickness of each part of the stem
    float thickness = INITAL_STEM_THICKNESS;
   
    for (int i=1; i<points.size(); i++) {
      
      // Set the colours
      float r = (COLOUR_R-255)*livelihood/100 + 255;
      float g = (COLOUR_G-255)*livelihood/100 + 255;
      float b = (COLOUR_B-255)*livelihood/100 + 255;
      stroke(r,g,b,255);
      
      // Draw the stem part
      strokeWeight(thickness);
      Point prevPoint = (Point) points.get(i-1);
      Point currPoint = (Point) points.get(i);
      line(prevPoint.x - CAMERA_POSITION.x + 400, prevPoint.y - CAMERA_POSITION.y + 210, 
           currPoint.x - CAMERA_POSITION.x + 400, currPoint.y - CAMERA_POSITION.y + 210);
      
      // Update the thickness
      thickness -= STEM_SHRINK_RATE;
      
      // This flower is dead
      if (thickness <= 1) {
        thickness = 0;
        livelihood -= 0.1f;
      }
    }
    
    // Draw the child (if any)
    if (child != null) {
      child.frame();
    }
  }
}
// ------------------------------------------------------------------------------
// Springs - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// HScroller.java
// A GUI element used to adjust various settings.
// ------------------------------------------------------------------------------

class HScroller
{
  float min;
  float max;
  float def;
  float x;
  float y;
  float xpos;
  float w;
  float scale;
  float value;
  boolean locked;
  
  HScroller( float Min, float Max, float Default, float X, float Y, float W )
  {
    min = Min;
    max = Max;
    x = X;
    y = Y;
    w = W;
    
    scale = ( max - min ) / w;
    def = ( Default - min ) / scale;
    xpos = def;
  }
  
  public float update()
  {
    drag();
    display();
    readValue();
    
    return value;
  }
  
  public void drag()
  {
    if ( mousePressed && over() )
    {
      locked = true;
    }
    if ( !mousePressed )
    {
      locked = false;
    }
    if ( locked )
    {
      assignPos(mouseX-x);
    }
  }
  
  public void assignPos( float X )
  {
    xpos = X;
    
    if ( X > w )
    {
      xpos = w;
    }
    if ( X < 0 )
    {
      xpos = 0;
    }
  }
  
  public void readValue()
  {
    value = xpos*scale + min;
  }
  
  public void display()
  {
    noSmooth();
    stroke(200,200,200);
    fill(230,230,230);
    strokeWeight(1);
    line(x,y,x+w,y);
    line(x+def, y-2, x+def, y+2);
    rectMode(CENTER);
    stroke(100,100,100);
    rect(x+xpos, y, 5, 5);
    smooth();
  }
  
  public boolean over() 
  { 
    if( mouseX > x - 10 && mouseX < x + w + 10 && mouseY > y - 10 && mouseY < y + 10 ) 
    { 
      return true; 
    } 
    else
    { 
      return false; 
    } 
  }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Point.java
// 2D Cartesian point.
// ------------------------------------------------------------------------------

class Point {
  
    /**
     * The actual coordinates.
     */
    float x;
    float y;

    /**
     * Constructor. Initialises
     * the point to the origin.
     */
    public Point() {
	x = 0;
	y = 0;
    }
    
    /**
     * Custom constructor.
     */
    Point(float X, float Y) {
	x = X;
	y = Y;
    }
    
    /**
     * Translate the point (addition)
     */
    public Point add(Vector V) {
	return new Point(x+V.a, y+V.b);
    }
    
    /**
     * Translate the point (subtraction)
     */
    public Point sub(Vector V) {
	return new Point(x-V.a, y-V.b);
    }
    
    /**
     * Multiplies the points coordinates by
     * a constant.
     */
    public Point mul(float K) {
	return new Point(x*K, y*K);
    }
    
    /**
     * Determines the equality of two points.
     * 
     * @return - the truth!
     */
    public boolean equals(Point P) {
	return (x == P.x && y == P.y);
    }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Point.java
// 2D Cartesian vector.
// ------------------------------------------------------------------------------

public class Vector {

    /**
     * The actual components.
     */
    float a;
    float b;
    
    /**
     * Constructor. Initialises
     * the vector to the O vector.
     */
    Vector() {
	a = 0;
	b = 0;
    }
    
    /**
     * Custom constructor.
     */
    Vector(float A, float B) {
	a = A;
	b = B;
    }
    
    /**
     * Custom constructor.
     */
    Vector(double A, double B) {
	a = (float) A;
	b = (float) B;
    }
    
    /** 
     * Creates a vector pointing from the
     * origin to the specified point.
     */
    Vector(Point P) {
	a = P.x;
	b = P.y;
    }
    
    /**
     * Creates a vector that points from 
     * the first point to the second point.
     */
    Vector(Point P1, Point P2) {
	a = P2.x - P1.x;
	b = P2.y - P1.y;
    }
    
    /**
     * Creates a vector that points in the
     * specified orientation.
     */
    Vector(float O) {
	a = (float) Math.cos(O);
	b = (float) Math.sin(O);
    }
    
    /**
     * Multiplies the vector by a constant
     */
    public Vector mul(float k) {
	return new Vector(a*k, b*k);
    }
    
    /**
     * Divides the vector by a constant. Beware
     * of passing zeroes. You WILL get nasty
     * errors :)
     */
    public Vector div(float k) {
	return new Vector(a/k, b/k);
    }
    
    /**
     * Adds another vector to this one.
     */
    public Vector add(Vector V) {
	return new Vector(a+V.a, b+V.b);
    }
    
    /**
     * Subtracts another vector from this one.
     */
    public Vector sub(Vector V) {
	return new Vector(a-V.a, b-V.b);
    }
    
    /**
     * Returns the length of the vector.
     */
    public float length() {
	return (float) Math.sqrt(Math.pow(a,2)+Math.pow(b, 2));
    }
    
    /**
     * Computes the dot product.
     */
    public float dot(Vector V) {
	return a*V.a + b*V.b;
    }

    /**
     * Returns a normal vector
     * to the current one.
     */
    public Vector normal() {
	return new Vector(-b, a);
    }
    
    /**
     * Returns the vector's orientation.
     */
    public float orientation() {
	return (float) Math.atan2(b,a);
    }
    
    /**
     * Rotates the vector by a certain amount.
     */
    public Vector rotate(float theta) {
	Vector R = new Vector();
	R.a = (float) (+ Math.cos(theta)*a - Math.sin(theta)*b);
	R.b = (float) (+ Math.sin(theta)*a + Math.cos(theta)*b);
	return R;
    }
}

// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// World.java
// The class that simulates the flowers and all the particles for the applet.
// ------------------------------------------------------------------------------

class World {
  
  ArrayList flowers = new ArrayList();          // The list of flowers
  
  /**
   * Executes a frame of the world.
   */
  public void frame() {
    
    // Update the global colour variables
    updateColors();
    
    // The list of flowers is empty
    if (flowers.size() == 0) {
      
      // Create a new flower and add it
      Flower newFlower = new Flower(CAMERA_POSITION, CAMERA_ORIENTATION);
      newFlower.primary = true;
      flowers.add(newFlower);
    }
    
    // Update the flowers
    CAMERA_UPDATED = false;
    for (int i=0; i<flowers.size(); i++) {

      // Update the flower and find out whether its dead or not
      boolean alive = ((Flower) flowers.get(i)).frame();
      
      // Its dead -- kill it off
      if (!alive) {
        Flower child = ((Flower) flowers.get(i)).child;
        boolean primary = ((Flower) flowers.get(i)).primary;
        if (child != null) {
          child.primary = primary;
          flowers.remove(i);
          flowers.add(i, child);
        }
        else {
          flowers.remove(i);
        }
      }
    }
    
    // We've run out of primaries
    if (!CAMERA_UPDATED) {
      ((Flower) flowers.get(0)).primary = true;
    }
  }
  
  /**
   * Updates the global colour variables
   */
  public void updateColors() {
    if (Math.abs(COLOUR_R-COLOUR_R_T) <= 2 || Math.abs(COLOUR_G-COLOUR_G_T) <= 2 || Math.abs(COLOUR_B-COLOUR_B_T) <= 2) {
      COLOUR_R_T = (float) (00 + Math.random()*255);
      COLOUR_G_T = (float) (00 + Math.random()*230);
      COLOUR_B_T = (float) (00 + Math.random()*230);
    }
    
    COLOUR_R += (COLOUR_R_T-COLOUR_R)/500;
    COLOUR_G += (COLOUR_G_T-COLOUR_G)/500;
    COLOUR_B += (COLOUR_B_T-COLOUR_B)/500;
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "GrowthV2" });}}