// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Shader.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// Global variables
Point    CAMERA_POSITION;
Vector   CAMERA_VELOCITY;
Vector   CAMERA_ORIENTATION;
boolean  CAMERA_UPDATED;
World    WORLD;

// The scrollers
HScroller Scroller1;
HScroller Scroller2;
HScroller Scroller3;
HScroller Scroller4;
HScroller Scroller5;

// Settings
float INITAL_STEM_THICKNESS         = 5.00;
float STEM_SHRINK_RATE              = 0.05;
float STEM_PART_LENGTH              = 2.00;
float STEM_ROTATION                 = 35.0;
float STEM_BRANCH_RATE              = 0.10;
float STEM_BENDING_INERTIA          = 1.00;
float STEM_BENDING_DAMPING          = 0.00;
float COLOUR_R                      = 0.00;
float COLOUR_G                      = 0.00;
float COLOUR_B                      = 0.00;
float COLOUR_R_T                    = 203.00;
float COLOUR_G_T                    = 10.00;
float COLOUR_B_T                    = 28.00;

/**
 * Gets executed once when the program
 * starts.
 */
void setup() {
  
  // Set the size and the framerate
  // and make sure all graphics are
  // smoothed out
  size(800,420);
  frameRate(30);
  smooth();
  
  // Set the proce55ing settings
  ellipseMode(CENTER);
  
  // Initialise the experiment's global variables
  CAMERA_POSITION = new Point(400, 210);
  CAMERA_VELOCITY = new Vector(0,0);
  CAMERA_ORIENTATION = new Vector(0,1);
  WORLD = new World();
  
  // Create the scrollers
  Scroller1 = new HScroller(5, 90, 35, 700, 290, 50);
  Scroller2 = new HScroller(0.0, 1.0, 1.0, 700, 310, 50);
  Scroller3 = new HScroller(0.0, 1.0, 0.0, 700, 330, 50);
  Scroller4 = new HScroller(2.0, 10, 2.0, 700, 350, 50);
  Scroller5 = new HScroller(0.0, 0.3, 0.1, 700, 370, 50);
}

/**
 * Gets executed every frame.
 */
void draw() {
  
  // Draw a purdy background
  background(255,255,255);
  
  // Update the variables
  STEM_ROTATION         = Scroller1.update();
  STEM_BENDING_INERTIA  = Scroller2.update();
  STEM_BENDING_DAMPING  = Scroller3.update();
  STEM_PART_LENGTH      = Scroller4.update();
  STEM_BRANCH_RATE      = Scroller5.update();
  
  // Execute a frame of the world
  WORLD.frame();
}

