// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Point.java
// 2D Cartesian vector.
// ------------------------------------------------------------------------------

public class Vector {

    /**
     * The actual components.
     */
    float a;
    float b;
    
    /**
     * Constructor. Initialises
     * the vector to the O vector.
     */
    Vector() {
	a = 0;
	b = 0;
    }
    
    /**
     * Custom constructor.
     */
    Vector(float A, float B) {
	a = A;
	b = B;
    }
    
    /**
     * Custom constructor.
     */
    Vector(double A, double B) {
	a = (float) A;
	b = (float) B;
    }
    
    /** 
     * Creates a vector pointing from the
     * origin to the specified point.
     */
    Vector(Point P) {
	a = P.x;
	b = P.y;
    }
    
    /**
     * Creates a vector that points from 
     * the first point to the second point.
     */
    Vector(Point P1, Point P2) {
	a = P2.x - P1.x;
	b = P2.y - P1.y;
    }
    
    /**
     * Creates a vector that points in the
     * specified orientation.
     */
    Vector(float O) {
	a = (float) Math.cos(O);
	b = (float) Math.sin(O);
    }
    
    /**
     * Multiplies the vector by a constant
     */
    Vector mul(float k) {
	return new Vector(a*k, b*k);
    }
    
    /**
     * Divides the vector by a constant. Beware
     * of passing zeroes. You WILL get nasty
     * errors :)
     */
    Vector div(float k) {
	return new Vector(a/k, b/k);
    }
    
    /**
     * Adds another vector to this one.
     */
    Vector add(Vector V) {
	return new Vector(a+V.a, b+V.b);
    }
    
    /**
     * Subtracts another vector from this one.
     */
    Vector sub(Vector V) {
	return new Vector(a-V.a, b-V.b);
    }
    
    /**
     * Returns the length of the vector.
     */
    float length() {
	return (float) Math.sqrt(Math.pow(a,2)+Math.pow(b, 2));
    }
    
    /**
     * Computes the dot product.
     */
    float dot(Vector V) {
	return a*V.a + b*V.b;
    }

    /**
     * Returns a normal vector
     * to the current one.
     */
    Vector normal() {
	return new Vector(-b, a);
    }
    
    /**
     * Returns the vector's orientation.
     */
    float orientation() {
	return (float) Math.atan2(b,a);
    }
    
    /**
     * Rotates the vector by a certain amount.
     */
    Vector rotate(float theta) {
	Vector R = new Vector();
	R.a = (float) (+ Math.cos(theta)*a - Math.sin(theta)*b);
	R.b = (float) (+ Math.sin(theta)*a + Math.cos(theta)*b);
	return R;
    }
}
